#! /bin/sh

sudo yum install -y \
    perl-CGI \
    perl-Clone \
    uuid-perl \
    perl-DBD-SQLite2 \
    perl-DBD-SQLite \
    perl-DBI \
    perl-JSON \
    perl-Excel-Writer-XLSX \
    perl-File-MimeInfo.noarch \
    perl-HTML-FromText \
    perl-HTML-Parser \
    perl-List-MoreUtils \
    perl-Parse-Yapp \
    perl-Spreadsheet-XLSX \
    perl-Text-CSV \
    perl-Text-CSV_XS \
    perl-Text-Markdown \
    perl-URI-Encode \
    sqlite \
\
    perl-Spreadsheet-ParseExcel \
    perl-Spreadsheet-WriteExcel \
    libxml2 \
;

# The last packages after the empty line are inexact matches.

# The following packages could not be found in Yum repositories and
# will have to be installed using 'cpan':

cpan -i \
    Config::Auto \
    URI::Escape \
    Spreadsheet::Read \
    Text::CSV::Encoded \
    Spreadsheet::ReadSXC \
;

# The 'xlsx2csv' has no package in CentOS-6.8, will have to install it
# manually :(

## sudo apt install -y libcgi-pm-perl
## sudo apt install -y libdata-uuid-perl
## sudo apt install -y libdbd-sqlite2-perl
## sudo apt install -y libdbd-sqlite3-perl
## sudo apt install -y libdbi-perl
## sudo apt install -y libexcel-writer-xlsx-perl
## sudo apt install -y libhtml-fromtext-perl
## sudo apt install -y libhtml-parser-perl
## sudo apt install -y liblist-moreutils-perl
## sudo apt install -y libparse-yapp-perl
## sudo apt install -y libspreadsheet-read-perl
## sudo apt install -y libspreadsheet-xlsx-perl
## sudo apt install -y libtext-csv-perl
## sudo apt install -y libtext-csv-encoded-perl
## sudo apt install -y libtext-csv-xs-perl
## sudo apt install -y libtext-markdown-perl
## sudo apt install -y liburi-encode-perl
## sudo apt install -y libxml2-utils
## sudo apt install -y sqlite3
## sudo apt install -y xlsx2csv
