html,
body {
  width: 100%;
  height: 100%;
}
body {
  background-color: #ecf0f1;
  margin: 0;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
  scrollbar-color: #d1d4d7 #dee0e2;
  scrollbar-width: thin;
}
div > p {
  text-align: center;
}
span {
  display: inline-block;
  vertical-align: middle;
  line-height: normal;
}
.card {
  background-color: #ecf0f1;
  min-width: 400px;
  min-height: 100%;
  display: block;
  /*specifically tell the browser to only scroll the card section*/
  overflow-y: scroll;
  /*rebeccapurple == #663399*/
}
/*For UX reasons you may want to hide the scrollbar, and still have a scrollable section.*/
.card::-webkit-scrollbar {
  display: none;
}
.card--content {
  background-color: ivory;
  min-height: 150px;
  margin: 10px;
  box-shadow: 3px 5px #e3e4e5;
  border: 1px solid #e3e4e5;
}
.box
{
    font-size: 1.2em;
    font-weight: bold;
    padding: 0 1em;
    display: flex;
    justify-content: center; /* align horizontal */
    align-items: center; /* align vertical */
}
/*.card--content {
  background-color: #e74c3c;
  min-width: 200px;
  margin: 5px;
  }*/
