#! /bin/sh

# Test generation of XLSX tables

test -z "${BIN_DIR}" && BIN_DIR=website/cgi-bin

SCRIPT_BASE=`basename $0 | sed "s/_[0-9]*\.sh$//"`

#BEGIN DEPEND------------------------------------------------------------------

INPUT_SCRIPT="${BIN_DIR}/${SCRIPT_BASE}"

#END DEPEND--------------------------------------------------------------------

set -ue

. tests/inputs/test1.domain.net-subresource.env
export REQUEST_METHOD=get

${INPUT_SCRIPT} 'filter=year>2000 AND year<1000' format=csv include=bibliography
