#! /bin/sh

sudo apt install -y gnumeric
sudo apt install -y libarchive-zip-perl
sudo apt install -y libauthen-oath-perl
sudo apt install -y libcgi-pm-perl
sudo apt install -y libclone-perl
sudo apt install -y libconfig-auto-perl
sudo apt install -y libdata-uuid-perl
sudo apt install -y libdata-validate-uri-perl
sudo apt install -y libdbd-mysql-perl
sudo apt install -y libdbd-sqlite2-perl
sudo apt install -y libdbd-sqlite3-perl
sudo apt install -y libdbi-perl
sudo apt install -y libexcel-writer-xlsx-perl
sudo apt install -y libexception-class-perl
sudo apt install -y libfile-libmagic-perl
sudo apt install -y libfile-mimeinfo-perl
sudo apt install -y libhash-merge-perl
sudo apt install -y libhtml-fromtext-perl
sudo apt install -y libhtml-parser-perl
sudo apt install -y libio-string-perl
sudo apt install -y libjson-perl
sudo apt install -y liblist-moreutils-perl
sudo apt install -y libparams-validate-perl
sudo apt install -y libparse-yapp-perl
sudo apt install -y libspreadsheet-read-perl
sudo apt install -y libspreadsheet-readsxc-perl
sudo apt install -y libspreadsheet-wright-perl
sudo apt install -y libspreadsheet-xlsx-perl
sudo apt install -y libtext-csv-perl
sudo apt install -y libtext-csv-encoded-perl
sudo apt install -y libtext-csv-xs-perl
sudo apt install -y libtext-markdown-perl
sudo apt install -y libtext-sprintfn-perl
sudo apt install -y libtry-tiny-perl
sudo apt install -y liburi-encode-perl
sudo apt install -y liburl-encode-perl
sudo apt install -y libwww-perl
sudo apt install -y libxml2-utils
sudo apt install -y sqlite3
sudo apt install -y xlsx2csv
sudo apt install -y xmlstarlet
sudo apt install -y zip
