#! /bin/sh

# Test generation of XLSX tables

test -z "${BIN_DIR}" && BIN_DIR=website/cgi-bin

SCRIPT_BASE=`basename $0 | sed "s/_[0-9]*\.sh$//"`

#BEGIN DEPEND------------------------------------------------------------------

INPUT_SCRIPT="${BIN_DIR}/${SCRIPT_BASE}"

#END DEPEND--------------------------------------------------------------------

set -ue

export REQUEST_URI='/~saulius/restful/website/test1.db/bibliography?rows=6'
export REQUEST_METHOD=get
export HTTP_HOST=test1.domain.net

${INPUT_SCRIPT} filter=id%20%3C%2020
