#! /bin/sh

# These tools help development of the package and are used in some
# Makefile targets (e.g. 'make cover'), but are not strictly needed to
# build the package or run the tests:

sudo apt install -y doxygen
sudo apt install -y graphviz
sudo apt install -y libdevel-cover-perl
sudo apt install -y libdevel-nytprof-perl
sudo apt install -y libgetopt-long-descriptive-perl
sudo apt install -y liblog-log4perl-perl
sudo apt install -y libpod-pom-perl
sudo apt install -y libtext-asciitable-perl
