// Scroll into view for input fields that are blocked by the fixed header.

"use strict";

function clearView() {
    var elements = document.querySelectorAll('input,select,textarea');

    elements.forEach(function(element){
        element.addEventListener('invalid', function () {
            this.scrollIntoView(false);
        });
    });
}
