#! /bin/sh

cpan -i \
    Archive::Zip \
    Authen::OATH \
    CGI \
    Clone \
    Config::Auto \
    DBI \
    JSON \
    DBD::SQLite2 \
    DBD::SQLite \
    Data::UUID \
    Data::Validate::URI \
    Excel::Writer::XLSX \
    File::MimeInfo \
    Hash::Merge \
    HTML::FromText \
    HTML::Parser \
    IO::String \
    List::MoreUtils \
    Object::Lazy \
    Parse::Yapp \
    Spreadsheet::Read \
    Spreadsheet::ReadSXC \
    Spreadsheet::Wright \
    Spreadsheet::XLSX \
    Text::CSV \
    Text::CSV_XS \
    Text::CSV::Encoded \
    Text::Markdown \
    Text::sprintfn \
    URI::Encode \
    URI::Escape \
    URL::Encode \
;
